import net.java.games.jogl.GL;
import net.java.games.jogl.GLDrawable; /**


 * <p>Title: Foundations of 3D Graphics Programming : Using JOGL and Java3D </p>
 *
 * <p>Description: a point bounces in a circle ...</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: George Mason University</p>
 *
 * @author Dr. Jim X. Chen
 * @version 1.0
 */
public class HW1_2_PointInCircle extends
    HW1_1_PointOnCirlcle {
  static final int pnum = 100; // number of points

  double direction[][] = new double[pnum][3]; // directions
  double point[][] = new double[pnum][3]; // points


  public HW1_2_PointInCircle() {

    for (int i = 0; i<pnum; i++) {
      direction[i][0] = Math.random();
      direction[i][1] = Math.random();
      direction[i][2] = 0;
      normalize(direction[i]); // make it a unit vector

      // generate a point inside the circle
      do {
        point[i][0] = Math.random()*WIDTH;
        point[i][1] = Math.random()*HEIGHT;
        point[i][2] = 0;
      } while (r<distance(cx, cy, point[i][0], point[i][1]));
    }
  }


  // Called for OpenGL rendering every reshape
  public void display(GLDrawable drawable) {

    cx = WIDTH/2;
    cy = HEIGHT/2;
    r = WIDTH/3;
    delta = 4/(Math.PI*r);

    gl.glClear(GL.GL_COLOR_BUFFER_BIT);

    //draw a white circle
    gl.glColor3f(1.0f, 1.0f, 1.0f);
    gl.glPointSize(4);
    drawCircle(cx, cy, r);

    gl.glPointSize(6);
    for (int i = 0; i<pnum; i++) {
      // move in a direction
      point[i][0] = point[i][0]+direction[i][0];
      point[i][1] = point[i][1]+direction[i][1];

      // bounce when point on/outside the circle
      if (r<distance(cx, cy, point[i][0], point[i][1])) {

        // change the direction of the point around r
        bounceInCircle(point[i], direction[i]);
      }
      //draw a colored point in Circle
      if (i<pnum/3) {
        gl.glColor3f(i*3f/pnum, 0.0f, 0.9f);
      } else if (i<2*pnum/3) {
        gl.glColor3f(0, i*3f/(2f*pnum), 0.9f);
      } else {
        gl.glColor3f(0.9f, 0.9f, i/pnum);
      }
      drawPoint(point[i][0], point[i][1]);
    }
  }


  public void bounceInCircle(double point[],
                             double direction[]) {
    double oldDirection[] = new double[3];
    double rDirection[] = new double[3];
    double cos2t, sin2t, cost, sint;

    // rDirection: a vector from center to currt point
    rDirection[0] = point[0]-cx;
    rDirection[1] = point[1]-cy;
    rDirection[2] = 0;
    normalize(rDirection);

    for (int i = 0; i<3; i++) {
      oldDirection[i] = direction[i];
      point[i] -= 2*rDirection[i]; // so remain in circle
    }

    // dot product of the two unit vectors
    cost = rDirection[0]*direction[0]+
           rDirection[1]*direction[1]+
           rDirection[2]*direction[2];

    //cross product of the two unit vectors
    sint = direction[0]*rDirection[1] - rDirection[0]*direction[1];

    cos2t = 2*cost*cost-1;
    sin2t = 2*sint*cost;

    // -x' = x*cos2t - y*sin2t
    direction[0] = -oldDirection[0]*cos2t+
                   oldDirection[1]*sin2t;
    // -y' = xsin2t + y*cos2t
    direction[1] = -oldDirection[0]*sin2t-
                   oldDirection[1]*cos2t;
    direction[2] = 0;
    normalize(direction);
  }


  double distance(double x0, double y0, double x1,
                  double y1) {

    return (Math.sqrt((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0)));
  }


  private void normalize(double vector[]) {
    double d = Math.sqrt(vector[0]*vector[0]
                         +vector[1]*vector[1]
                         +vector[2]*vector[2]);

    if (d==0) {
      System.out.println("0 length vector: normalize().");
      return;
    }
    vector[0] /= d;
    vector[1] /= d;
    vector[2] /= d;
  }


  public static void main(String[] args) {
    HW1_2_PointInCircle f = new HW1_2_PointInCircle();

    f.setTitle("HW1_2 - points bounce in a circle");
    f.setSize(WIDTH, HEIGHT);
    f.setVisible(true);

  }
}
